@@ -28,28 +28,134 @@
 #include <vncdisplay.h>
 
 #include <vinagre/vinagre-debug.h>
-#include <vinagre/vinagre-utils.h>
+#include <vinagre/vinagre-prefs.h>
 
 #include "vinagre-vnc-plugin.h"
 #include "vinagre-vnc-connection.h"
 #include "vinagre-vnc-tab.h"
+#include "vinagre-vnc-listener-dialog.h"
+#include "vinagre-vnc-listener.h"
 
 #define VINAGRE_VNC_PLUGIN_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), VINAGRE_TYPE_VNC_PLUGIN, VinagreVncPluginPrivate))
+#define WINDOW_DATA_KEY "VinagreVNCPluginWindowData"
 
 VINAGRE_PLUGIN_REGISTER_TYPE(VinagreVncPlugin, vinagre_vnc_plugin)
 
+typedef struct
+{
+  GtkActionGroup *ui_action_group;
+  guint ui_id;
+} WindowData;
+
+typedef struct
+{
+  VinagrePlugin *plugin;
+  VinagreWindow *window;
+} ActionData;
+
+static void
+free_window_data (WindowData *data)
+{
+  g_return_if_fail (data != NULL);
+
+  g_object_unref (data->ui_action_group);
+  g_slice_free (WindowData, data);
+}
+
+static void
+free_action_data (ActionData *data)
+{
+  g_return_if_fail (data != NULL);
+
+  g_slice_free (ActionData, data);
+}
+
+static void
+listening_cb (GtkAction *action, ActionData *action_data)
+{
+  vinagre_vnc_listener_dialog_show (action_data->window, action_data->plugin);
+}
+
+static GtkActionEntry action_entries[] =
+{
+  { "VNCListener",
+    NULL,
+    N_("_Reverse Connections..."),
+    NULL,
+    N_("Configure incoming VNC connections"),
+    G_CALLBACK (listening_cb)
+  }
+};
+
 static void
 impl_activate (VinagrePlugin *plugin,
                VinagreWindow *window)
 {
+  GtkActionGroup *action_group;
+  GtkUIManager *manager;
+  WindowData *data;
+  ActionData *action_data;
+  gboolean always;
+
   vinagre_debug_message (DEBUG_PLUGINS, "VinagreVncPlugin Activate");
+
+  data = g_slice_new (WindowData);
+  action_data = g_slice_new (ActionData);
+  action_data->window = window;
+  action_data->plugin = plugin;
+
+  action_group = vinagre_window_get_always_sensitive_action (window);
+  manager = vinagre_window_get_ui_manager (window);
+
+  data->ui_action_group = gtk_action_group_new ("VinagreVNCPluginActions");
+  gtk_action_group_set_translation_domain (data->ui_action_group, GETTEXT_PACKAGE);
+  gtk_action_group_add_actions_full (data->ui_action_group,
+				     action_entries,
+				     G_N_ELEMENTS (action_entries),
+				     action_data,
+				     (GDestroyNotify) free_action_data);
+  gtk_ui_manager_insert_action_group (manager,
+				      data->ui_action_group,
+				      -1);
+
+  data->ui_id = gtk_ui_manager_new_merge_id (manager);
+  gtk_ui_manager_add_ui (manager,
+			 data->ui_id,
+			 "/MenuBar/MachineMenu/MachineOps_1",
+			 "VNCListener",
+			 "VNCListener",
+			 GTK_UI_MANAGER_AUTO,
+			 TRUE);
+
+  g_object_set_data_full (G_OBJECT (window),
+			  WINDOW_DATA_KEY,
+			  data,
+			  (GDestroyNotify) free_window_data);
+
+  g_object_get (vinagre_prefs_get_default (),
+		"always-enable-listening", &always,
+		NULL);
+  if (always)
+    vinagre_vnc_listener_start (vinagre_vnc_listener_get_default ());
 }
 
 static void
 impl_deactivate  (VinagrePlugin *plugin,
                   VinagreWindow *window)
 {
+  GtkUIManager *manager;
+  WindowData *data;
+
   vinagre_debug_message (DEBUG_PLUGINS, "VinagreVncPlugin Deactivate");
+
+  manager = vinagre_window_get_ui_manager (window);
+  data = (WindowData *) g_object_get_data (G_OBJECT (window), WINDOW_DATA_KEY);
+  g_return_if_fail (data != NULL);
+
+  gtk_ui_manager_remove_ui (manager, data->ui_id);
+  gtk_ui_manager_remove_action_group (manager, data->ui_action_group);
+
+  g_object_set_data (G_OBJECT (window), WINDOW_DATA_KEY, NULL);
 }
 
 static void
@@ -234,20 +340,22 @@ impl_new_tab (VinagrePlugin *plugin,
 static GtkWidget *
 impl_get_connect_widget (VinagrePlugin *plugin, VinagreConnection *conn)
 {
-  GtkWidget *box, *check, *label;
+  GtkWidget *box, *check, *label, *combo, *depth_box;
   GtkTable  *table;
   gchar     *str;
+  gint      depth_profile;
 
-  box = gtk_vbox_new (TRUE, 0);
+  box = gtk_vbox_new (FALSE, 0);
 
   str = g_strdup_printf ("<b>%s</b>", _("VNC Options"));
   label = gtk_label_new (str);
   g_free (str);
   gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-  gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 0);
+  gtk_misc_set_padding (GTK_MISC (label), 0, 6);
+  gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);
 
-  table = GTK_TABLE (gtk_table_new (2, 2, FALSE));
+  table = GTK_TABLE (gtk_table_new (4, 2, FALSE));
   label = gtk_label_new ("  ");
   gtk_table_attach (table, label, 0, 1, 0, 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
 
@@ -265,7 +373,37 @@ impl_get_connect_widget (VinagrePlugin *plugin, VinagreConnection *conn)
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check),
 				  vinagre_vnc_connection_get_scaling (VINAGRE_VNC_CONNECTION (conn)));
 
-  gtk_box_pack_start (GTK_BOX (box), GTK_WIDGET (table), TRUE, TRUE, 0);
+  check = gtk_check_button_new_with_mnemonic (_("_Use JPEG Compression"));
+  gtk_widget_set_tooltip_text (check, _("This might not work on all sort of VNC servers"));
+  g_object_set_data (G_OBJECT (box), "lossy", check);
+  gtk_table_attach_defaults (table, check, 1, 2, 2, 3);
+  if (VINAGRE_IS_VNC_CONNECTION (conn))
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check),
+				  vinagre_vnc_connection_get_lossy_encoding (VINAGRE_VNC_CONNECTION (conn)));
+
+  depth_box = gtk_hbox_new (FALSE, 4);
+  label = gtk_label_new_with_mnemonic (_("_Depth Color:"));
+  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+  gtk_box_pack_start (GTK_BOX (depth_box), GTK_WIDGET (label), FALSE, FALSE, 0);
+
+  if (VINAGRE_IS_VNC_CONNECTION (conn))
+    depth_profile = vinagre_vnc_connection_get_depth_profile (VINAGRE_VNC_CONNECTION (conn));
+  else
+    depth_profile = 0;
+
+  combo = gtk_combo_box_new_text ();
+  gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Use Server Settings"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("True Color (24 bits)"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("High Color (16 bits)"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Low Color (8 bits)"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Ultra Low Color (3 bits)"));
+  gtk_combo_box_set_active (GTK_COMBO_BOX (combo), depth_profile);
+  g_object_set_data (G_OBJECT (box), "depth_combo", combo);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label), combo);
+  gtk_box_pack_start (GTK_BOX (depth_box), GTK_WIDGET (combo), FALSE, FALSE, 0);
+  gtk_table_attach_defaults (table, depth_box, 1, 2, 3, 4);
+
+  gtk_box_pack_start (GTK_BOX (box), GTK_WIDGET (table), FALSE, FALSE, 0);
   return box;
 }
 
