@@ -103,6 +103,12 @@ view_original_size_cb (GtkAction *action, VinagreVncTab *vnc_tab)
   vinagre_vnc_tab_original_size (vnc_tab);
 }
 
+static void
+view_refresh_cb (GtkAction *action, VinagreVncTab *vnc_tab)
+{
+  vnc_display_request_update (VNC_DISPLAY (vnc_tab->priv->vnc));
+}
+
 const static GSList *
 vnc_get_connected_actions (VinagreTab *tab)
 {
@@ -240,8 +246,8 @@ static void
 open_vnc (VinagreVncTab *vnc_tab)
 {
   gchar      *host, *port_str;
-  gint       port, shared;
-  gboolean   scaling;
+  gint       port, shared, fd, depth_profile;
+  gboolean   scaling, success, lossy_encoding;
   VncDisplay *vnc = VNC_DISPLAY (vnc_tab->priv->vnc);
   VinagreTab *tab = VINAGRE_TAB (vnc_tab);
 
@@ -250,6 +256,9 @@ open_vnc (VinagreVncTab *vnc_tab)
 		"host", &host,
 		"scaling", &scaling,
 		"shared", &shared,
+		"fd", &fd,
+		"depth-profile", &depth_profile,
+		"lossy-encoding", &lossy_encoding,
 		NULL);
 
   port_str = g_strdup_printf ("%d", port);
@@ -260,8 +269,15 @@ open_vnc (VinagreVncTab *vnc_tab)
 
   vnc_display_set_shared_flag (vnc, shared);
   vnc_display_set_force_size (vnc, !scaling);
+  vnc_display_set_depth (vnc, depth_profile);
+  vnc_display_set_lossy_encoding (vnc, lossy_encoding);
 
-  if (vnc_display_open_host (vnc, host, port_str))
+  if (fd > 0)
+    success = vnc_display_open_fd (vnc, fd);
+  else
+    success = vnc_display_open_host (vnc, host, port_str);
+
+  if (success)
     gtk_widget_grab_focus (GTK_WIDGET (vnc));
   else
     vinagre_utils_show_error (NULL,
@@ -375,12 +391,12 @@ vnc_initialized_cb (VncDisplay *vnc, VinagreVncTab *vnc_tab)
 
   gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (vnc_tab->priv->scaling_action), scaling);
   gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (vnc_tab->priv->viewonly_action), view_only);
-  vnc_display_set_pointer_local (VNC_DISPLAY(vnc_tab->priv->vnc), TRUE);
-  vnc_display_set_keyboard_grab (VNC_DISPLAY(vnc_tab->priv->vnc), TRUE);
-  vnc_display_set_pointer_grab (VNC_DISPLAY(vnc_tab->priv->vnc), TRUE);
+  vnc_display_set_pointer_local (vnc, TRUE);
+  vnc_display_set_keyboard_grab (vnc, TRUE);
+  vnc_display_set_pointer_grab (vnc, TRUE);
 
   vinagre_vnc_connection_set_desktop_name (VINAGRE_VNC_CONNECTION (conn),
-					   vnc_display_get_name (VNC_DISPLAY (vnc_tab->priv->vnc)));
+					   vnc_display_get_name (vnc));
 
   name = vinagre_connection_get_best_name (conn);
   label = g_object_get_data (G_OBJECT (tab), "label");
@@ -429,7 +445,7 @@ ask_credential (VinagreVncTab *vnc_tab,
   VinagreTab      *tab = VINAGRE_TAB (vnc_tab);
   VinagreConnection *conn = vinagre_tab_get_conn (tab);
 
-  xml = vinagre_utils_get_builder ();
+  xml = vinagre_utils_get_builder (NULL, NULL);
 
   password_dialog = GTK_WIDGET (gtk_builder_get_object (xml, "auth_required_dialog"));
   gtk_window_set_transient_for (GTK_WINDOW(password_dialog),
@@ -673,6 +689,19 @@ create_connected_actions (VinagreVncTab *tab)
   list = g_slist_append (list, a);
   tab->priv->original_size_action = a->action;
 
+  /* View->Refresh */
+  a = g_new (VinagreTabUiAction, 1);
+  a->paths = g_new (gchar *, 2);
+  a->paths[0] = g_strdup ("/MenuBar/ViewMenu");
+  a->paths[1] = NULL;
+  a->action = gtk_action_new ("VNCViewRefresh",
+			      _("_Refresh Screen"),
+			      _("Requests an update of the screen"),
+			      "gtk-refresh");
+  gtk_action_set_icon_name (a->action, "gtk-refresh");
+  g_signal_connect (a->action, "activate", G_CALLBACK (view_refresh_cb), tab);
+  list = g_slist_append (list, a);
+
   return list;
 }
 
@@ -761,9 +790,8 @@ setup_toolbar (VinagreVncTab *vnc_tab)
   vnc_tab->priv->viewonly_button = button;
 
   /* Send Ctrl-alt-del */
-  button = GTK_WIDGET (gtk_tool_button_new (NULL, NULL));
+  button = GTK_WIDGET (gtk_tool_button_new (NULL, _("Send Ctrl-Alt-Del")));
   gtk_tool_button_set_icon_name (GTK_TOOL_BUTTON (button), "preferences-desktop-keyboard-shortcuts");
-  gtk_tool_button_set_label (GTK_TOOL_BUTTON (button), _("Send Ctrl-Alt-Del"));
   gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (button), _("Send Ctrl-Alt-Del"));
   g_signal_connect (button, "clicked", G_CALLBACK (cad_button_clicked), vnc_tab);
   gtk_widget_show (GTK_WIDGET (button));
@@ -790,10 +818,52 @@ vnc_tab_clipboard_cb (GtkClipboard *cb, GdkEvent *event, VinagreVncTab *vnc_tab)
   g_free (text);
 }
 
+/*
+ * Called when the main container widget's size has been set.
+ * It attempts to fit the VNC widget into this space while
+ * maintaining aspect ratio
+ *
+ * Code borrowed from from virt-viewer, thanks Daniel Berrange :)
+ */
+static void
+vnc_tab_resize_align (GtkWidget *widget,
+		      GtkAllocation *alloc,
+		      VinagreVncTab *vnc_tab)
+{
+  double desktopAspect = (double)vnc_display_get_width (VNC_DISPLAY (vnc_tab->priv->vnc)) / (double)vnc_display_get_height (VNC_DISPLAY (vnc_tab->priv->vnc));
+  double scrollAspect = (double)alloc->width / (double)alloc->height;
+  int height, width;
+  GtkAllocation child;
+  int dx = 0, dy = 0;
+
+  if (!vnc_display_is_open (VNC_DISPLAY (vnc_tab->priv->vnc)))
+    return;
+
+  if (scrollAspect > desktopAspect)
+    {
+      width = alloc->height * desktopAspect;
+      dx = (alloc->width - width) / 2;
+      height = alloc->height;
+    }
+  else
+    {
+      width = alloc->width;
+      height = alloc->width / desktopAspect;
+      dy = (alloc->height - height) / 2;
+    }
+
+  child.x = alloc->x + dx;
+  child.y = alloc->y + dy;
+  child.width = width;
+  child.height = height;
+  gtk_widget_size_allocate(vnc_tab->priv->vnc, &child);
+}
+
 static void
 vinagre_vnc_tab_init (VinagreVncTab *vnc_tab)
 {
   GtkClipboard *cb;
+  GtkWidget *align;
 
   vnc_tab->priv = VINAGRE_VNC_TAB_GET_PRIVATE (vnc_tab);
   vnc_tab->priv->clipboard_str = NULL;
@@ -802,7 +872,13 @@ vinagre_vnc_tab_init (VinagreVncTab *vnc_tab)
 
   /* Create the vnc widget */
   vnc_tab->priv->vnc = vnc_display_new ();
-  vinagre_tab_add_view (VINAGRE_TAB (vnc_tab), vnc_tab->priv->vnc);
+  align = gtk_alignment_new (0.5, 0.5, 1, 1);
+
+  g_signal_connect(align, "size-allocate",
+		  G_CALLBACK (vnc_tab_resize_align), vnc_tab);
+  gtk_container_add (GTK_CONTAINER (align), vnc_tab->priv->vnc);
+
+  vinagre_tab_add_view (VINAGRE_TAB (vnc_tab), align);
 
   g_signal_connect (vnc_tab->priv->vnc,
 		    "vnc-connected",
