@@ -48,6 +48,7 @@ typedef struct {
   GtkWidget *find_button;
   GtkWidget *fullscreen_check;
   GtkWidget *plugin_box;
+  GtkWidget *connect_button;
 } VinagreConnectDialog;
 
 enum {
@@ -220,14 +221,23 @@ saved_history (void)
 }
 
 static void
-setup_combo (GtkWidget *combo)
+control_connect_button (GtkEditable *entry, VinagreConnectDialog *dialog)
+{
+  gtk_widget_set_sensitive (dialog->connect_button,
+			    gtk_entry_get_text_length (GTK_ENTRY (entry)) > 0);
+}
+
+static void
+setup_combo (VinagreConnectDialog *dialog)
 {
   GtkListStore *store;
   GtkTreeIter   iter;
   GtkEntryCompletion *completion;
   GPtrArray    *history;
   gint          i, size;
+  GtkEntry     *entry;
 
+  entry = GTK_ENTRY (gtk_bin_get_child (GTK_BIN (dialog->host_entry)));
   store = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING);
 
   history = saved_history ();
@@ -244,19 +254,20 @@ setup_combo (GtkWidget *combo)
     }
   g_ptr_array_free (history, TRUE);
 
-  gtk_combo_box_set_model (GTK_COMBO_BOX (combo),
+  gtk_combo_box_set_model (GTK_COMBO_BOX (dialog->host_entry),
 			   GTK_TREE_MODEL (store));
-  gtk_combo_box_entry_set_text_column (GTK_COMBO_BOX_ENTRY (combo),
+  gtk_combo_box_entry_set_text_column (GTK_COMBO_BOX_ENTRY (dialog->host_entry),
 				       0);
 
   completion = gtk_entry_completion_new ();
   gtk_entry_completion_set_model (completion, GTK_TREE_MODEL (store));
   gtk_entry_completion_set_text_column (completion, 0);
   gtk_entry_completion_set_inline_completion (completion, TRUE);
-  gtk_entry_set_completion (GTK_ENTRY(gtk_bin_get_child(GTK_BIN(combo))), completion);
+  gtk_entry_set_completion (entry, completion);
   g_object_unref (completion);
 
-  gtk_entry_set_activates_default (GTK_ENTRY(gtk_bin_get_child(GTK_BIN(combo))), TRUE);
+  gtk_entry_set_activates_default (entry, TRUE);
+  g_signal_connect (entry, "changed", G_CALLBACK (control_connect_button), dialog);
 }
 
 static void
@@ -341,10 +352,14 @@ vinagre_connect_find_button_cb (GtkButton            *button,
   if (gtk_dialog_run(GTK_DIALOG(d)) == GTK_RESPONSE_ACCEPT)
     {
       gchar *tmp;
+      char a[AVAHI_ADDRESS_STR_MAX];
 
+      avahi_address_snprint (a,
+			     sizeof(a),
+			     aui_service_dialog_get_address (AUI_SERVICE_DIALOG (d)));
       tmp = g_strdup_printf ("%s::%d",
-			     aui_service_dialog_get_host_name(AUI_SERVICE_DIALOG(d)),
-			     aui_service_dialog_get_port(AUI_SERVICE_DIALOG(d)));
+			     a,
+			     aui_service_dialog_get_port (AUI_SERVICE_DIALOG (d)));
 
       gtk_entry_set_text (GTK_ENTRY (gtk_bin_get_child (GTK_BIN (dialog->host_entry))),
 			  tmp);
@@ -363,7 +378,7 @@ VinagreConnection *vinagre_connect (VinagreWindow *window)
   gint                  result;
   VinagreConnectDialog  dialog;
 
-  dialog.xml = vinagre_utils_get_builder ();
+  dialog.xml = vinagre_utils_get_builder (NULL, NULL);
 
   dialog.dialog = GTK_WIDGET (gtk_builder_get_object (dialog.xml, "connect_dialog"));
   gtk_window_set_transient_for (GTK_WINDOW (dialog.dialog), GTK_WINDOW (window));
@@ -374,9 +389,10 @@ VinagreConnection *vinagre_connect (VinagreWindow *window)
   dialog.find_button = GTK_WIDGET (gtk_builder_get_object (dialog.xml, "find_button"));
   dialog.fullscreen_check = GTK_WIDGET (gtk_builder_get_object (dialog.xml, "fullscreen_check"));
   dialog.plugin_box = GTK_WIDGET (gtk_builder_get_object (dialog.xml, "plugin_options_connect_vbox"));
+  dialog.connect_button = GTK_WIDGET (gtk_builder_get_object (dialog.xml, "connect_button"));
 
   setup_protocol (&dialog);
-  setup_combo (dialog.host_entry);
+  setup_combo (&dialog);
 
 #ifdef VINAGRE_ENABLE_AVAHI
   g_signal_connect (dialog.find_button,
